//const path = require('path')
const { exec } = require("child_process");
const { ipcRenderer, app, shell } = require('electron');

//var shell = remote.shell;

const path = require("path");
const executablePathWin = path.join(__dirname, '../bin/win/yt-dlp.exe');
const executablePathMacOs = path.join(__dirname, '../bin/macOS/./yt-dlp_macos');
const ffmpegPathMacOs     = path.join(__dirname, '../bin/macOS/./ffmpeg');
const executablePathLinux = path.join(__dirname, '../bin/linux/./yt-dlp_linux');
const ffmpegPathLinux     = path.join(__dirname, '../bin/linux/./ffmpeg');
const fs = require("fs");

const elUrl = document.getElementById('js_url');
const elAudioOnly = document.getElementById('js_AudioOnly');
const elDownload = document.getElementById('js_download');

const elForm = document.getElementById('js_form');
const elLoading = document.getElementById('js_loading');

const webSiteButton = document.getElementById('js_openWebSite');

var workingState = false;

const UrlRegex = /^(https?:\/\/)?(www\.)?([a-zA-Z0-9\-]+\.)+[a-zA-Z]{2,}(\/[^\s]*)?$/;

let os;
if (navigator.platform.indexOf("Win") != -1) os = "Windows";
if (navigator.platform.indexOf("Mac") != -1) os = "macOs";
if (navigator.platform.indexOf("Lin") != -1) os = "Linux";

const COMMAND = {
    'createfolder': {
      'win': 'mkdir "%UserProfile%\\Videos\\Fulldl"',
      'macOs': 'mkdir ~/Movies/Fulldl',
      'linux': 'mkdir ~/Videos/Fulldl'
    },
    'openfolder': {
      'win': 'explorer.exe "%UserProfile%\\Videos\\Fulldl"',
      'macOs': 'open ~/Movies/Fulldl/',
      'linux': 'open ~/Videos/Fulldl/'
    },
    'openWebSite' : {
        'win': 'start https://fulldl.fr',
        'macOs': 'open https://fulldl.fr',
        'linux': 'open https://fulldl.fr'
    },
    'vidArgs' : {
        'win' : ' --merge-output-format mp4 -f \"bestvideo+bestaudio[ext=m4a]/best\" -o  %UserProfile%\\Videos\\Fulldl\\%(title)s.%(ext)s ',
        'linux' : ' --ffmpeg-location "' + ffmpegPathLinux + '"  --merge-output-format "mp4" "-f bestvideo[ext=mp4]+bestaudio[ext=m4a]/best" -o "~/Videos/Fulldl/%(title)s.%(ext)s" ',
        'macOs' : ' --ffmpeg-location "' + ffmpegPathMacOs + '"  --merge-output-format "mp4" "-f bestvideo[ext=mp4]+bestaudio[ext=m4a]/best" -o "~/Movies/Fulldl/%(title)s.%(ext)s" '
    },
    'audArgs' : {
        'win' : ' -x --audio-format mp3 -o %UserProfile%\\Videos\\Fulldl\\%(title)s.%(ext)s ',
        'linux' : ' --ffmpeg-location "' + ffmpegPathLinux + '" -x --audio-format "mp3" -o "~/Videos/Fulldl/%(title)s.%(ext)s" ',
        'macOs' : ' --ffmpeg-location "' + ffmpegPathMacOs + '" -x --audio-format "mp3" -o "~/Movies/Fulldl/%(title)s.%(ext)s" '
    }
  };

function isUrl(url)
{
    return UrlRegex.test(url);
}

elDownload.addEventListener('click', (e) =>
{
    e.preventDefault();

    if (workingState || elUrl.value == '' || !isUrl(elUrl.value))
        return;

    setWorking(true);
    download(elAudioOnly.checked, elUrl.value);
});

function setWorking(state)
{
    workingState = state;

    if (state)
    {
        if (!elForm.classList.contains('d-none'))
            elForm.classList.add('d-none');

        if (elLoading.classList.contains('d-none'))
            elLoading.classList.remove('d-none');
    }
    else
    {
        if (elForm.classList.contains('d-none'))
            elForm.classList.remove('d-none');

        if (!elLoading.classList.contains('d-none'))
            elLoading.classList.add('d-none');
    }
}

function download(audioOnly, url)
{

    let cmd;

    if (os == 'Windows')
        {
            cmd = '"' + executablePathWin + '"';
            if (audioOnly)
                cmd += COMMAND['audArgs']['win'];
            else
                cmd += COMMAND['vidArgs']['win'];
        } 
        else if( os == 'macOs')
        {
            cmd = '"' + executablePathMacOs + '"';
            if (audioOnly)
                cmd += COMMAND['audArgs']['macOs'];
            else
                cmd += COMMAND['vidArgs']['macOs'];
        }
        else if (os == 'Linux')
        {
            cmd = '"' + executablePathLinux + '"';
            if (audioOnly)
                cmd += COMMAND['audArgs']['linux'];
            else
                cmd += COMMAND['vidArgs']['linux'];
        }

    cmd += url;

    exec(cmd, (error, data, getter) =>
    {
        if (error)
        {
            alert("Error: " + error.message);
            setWorking(false);
            return;
        }

        if (getter)
        {
            //alert("Getter: " + data);
            setWorking(false);
            return;
        }
        setWorking(false);
    });
    if (os == 'Windows')
    {
        exec(COMMAND['createfolder']['win']);
        exec(COMMAND['openfolder']['win']);
    } 
    else if( os == 'macOs')
    {
        exec(COMMAND['createfolder']['macOs']);
        exec(COMMAND['openfolder']['macOs']);
    }
    else if ( os == 'Linux')
    {
        exec(COMMAND['createfolder']['linux']);
        exec(COMMAND['openfolder']['linux']);
    }
}


webSiteButton.addEventListener( 'click', ( )=> {

if (os == 'Windows')
    {
        exec(COMMAND['openWebSite']['win']);
    } 
    else if( os == 'macOs')
    {
        exec(COMMAND['openWebSite']['macOs']);
    }
    else if ( os == 'Linux')
    {
        exec(COMMAND['openWebSite']['linux']);
    }
 });
